@@ -3,7 +3,8 @@
 
     This file is part of qgpgme, the Qt API binding for gpgme
     Copyright (c) 2004,2005 Klarälvdalens Datakonsult AB
-    Copyright (c) 2016 Intevation GmbH
+    Copyright (c) 2016 by Bundesamt für Sicherheit in der Informationstechnik
+    Software engineering by Intevation GmbH
 
     QGpgME is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -51,18 +52,18 @@ const char QGpgME::QGpgMEBackend::SMIME[] = "SMIME";
 
 
 QGpgME::QGpgMEBackend::QGpgMEBackend()
-    : mCryptoConfig(0),
-      mOpenPGPProtocol(0),
-      mSMIMEProtocol(0)
+    : mCryptoConfig(nullptr),
+      mOpenPGPProtocol(nullptr),
+      mSMIMEProtocol(nullptr)
 {
     GpgME::initializeLibrary();
 }
 
 QGpgME::QGpgMEBackend::~QGpgMEBackend()
 {
-    delete mCryptoConfig; mCryptoConfig = 0;
-    delete mOpenPGPProtocol; mOpenPGPProtocol = 0;
-    delete mSMIMEProtocol; mSMIMEProtocol = 0;
+    delete mCryptoConfig; mCryptoConfig = nullptr;
+    delete mOpenPGPProtocol; mOpenPGPProtocol = nullptr;
+    delete mSMIMEProtocol; mSMIMEProtocol = nullptr;
 }
 
 QString QGpgME::QGpgMEBackend::name() const
@@ -162,7 +163,7 @@ QGpgME::Protocol *QGpgME::QGpgMEBackend::protocol(const char *name) const
     if (qstricmp(name, SMIME) == 0) {
         return smime();
     }
-    return 0;
+    return nullptr;
 }
 
 bool QGpgME::QGpgMEBackend::supportsProtocol(const char *name) const
@@ -175,7 +176,7 @@ const char *QGpgME::QGpgMEBackend::enumerateProtocols(int i) const
     switch (i) {
     case 0: return OpenPGP;
     case 1: return SMIME;
-    default: return 0;
+    default: return nullptr;
     }
 }
 
