@@ -1,7 +1,8 @@
 /* t-encrypt.cpp
 
     This file is part of qgpgme, the Qt API binding for gpgme
-    Copyright (c) 2016 Intevation GmbH
+    Copyright (c) 2016 by Bundesamt für Sicherheit in der Informationstechnik
+    Software engineering by Intevation GmbH
 
     QGpgME is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -28,6 +29,10 @@
     you do not wish to do so, delete this exception statement from
     your version.
 */
+#ifdef HAVE_CONFIG_H
+ #include "config.h"
+#endif
+
 #include <QDebug>
 #include <QTest>
 #include <QTemporaryDir>
@@ -35,13 +40,16 @@
 #include <QBuffer>
 #include "keylistjob.h"
 #include "encryptjob.h"
-#include "qgpgmeencryptjob.h"
+#include "signencryptjob.h"
+#include "signingresult.h"
+#include "encryptjob.h"
 #include "encryptionresult.h"
 #include "decryptionresult.h"
-#include "qgpgmedecryptjob.h"
+#include "decryptjob.h"
 #include "qgpgmebackend.h"
 #include "keylistresult.h"
 #include "engineinfo.h"
+#include "verifyopaquejob.h"
 #include "t-support.h"
 
 #define PROGRESS_TEST_SIZE 1 * 1024 * 1024
@@ -49,6 +57,23 @@
 using namespace QGpgME;
 using namespace GpgME;
 
+static bool decryptSupported()
+{
+    /* With GnuPG 2.0.x (at least 2.0.26 by default on jessie)
+     * the passphrase_cb does not work. So the test popped up
+     * a pinentry. So tests requiring decryption don't work. */
+    static auto version = GpgME::engineInfo(GpgME::GpgEngine).engineVersion();
+    if (version < "2.0.0") {
+        /* With 1.4 it just works */
+        return true;
+    }
+    if (version < "2.1.0") {
+        /* With 2.1 it works with loopback mode */
+        return false;
+    }
+    return true;
+}
+
 class EncryptionTest : public QGpgMETest
 {
     Q_OBJECT
@@ -64,29 +89,32 @@ private Q_SLOTS:
         std::vector<Key> keys;
         auto keylistresult = listjob->exec(QStringList() << QStringLiteral("alfa@example.net"),
                                           false, keys);
-        Q_ASSERT(!keylistresult.error());
-        Q_ASSERT(keys.size() == 1);
+        QVERIFY(!keylistresult.error());
+        QVERIFY(keys.size() == 1);
         delete listjob;
 
         auto job = openpgp()->encryptJob(/*ASCII Armor */true, /* Textmode */ true);
-        Q_ASSERT(job);
+        QVERIFY(job);
         QByteArray cipherText;
         auto result = job->exec(keys, QStringLiteral("Hello World").toUtf8(), Context::AlwaysTrust, cipherText);
         delete job;
-        Q_ASSERT(!result.error());
+        QVERIFY(!result.error());
         const auto cipherString = QString::fromUtf8(cipherText);
-        Q_ASSERT(cipherString.startsWith("-----BEGIN PGP MESSAGE-----"));
+        QVERIFY(cipherString.startsWith("-----BEGIN PGP MESSAGE-----"));
 
         /* Now decrypt */
-        auto ctx = Context::createForProtocol(OpenPGP);
+        if (!decryptSupported()) {
+            return;
+        }
+        auto decJob = openpgp()->decryptJob();
+        auto ctx = Job::context(decJob);
         TestPassphraseProvider provider;
         ctx->setPassphraseProvider(&provider);
         ctx->setPinentryMode(Context::PinentryLoopback);
-        auto decJob = new QGpgMEDecryptJob(ctx);
         QByteArray plainText;
         auto decResult = decJob->exec(cipherText, plainText);
-        Q_ASSERT(!result.error());
-        Q_ASSERT(QString::fromUtf8(plainText) == QStringLiteral("Hello World"));
+        QVERIFY(!decResult.error());
+        QVERIFY(QString::fromUtf8(plainText) == QStringLiteral("Hello World"));
         delete decJob;
     }
 
@@ -101,36 +129,36 @@ private Q_SLOTS:
         std::vector<Key> keys;
         auto keylistresult = listjob->exec(QStringList() << QStringLiteral("alfa@example.net"),
                                           false, keys);
-        Q_ASSERT(!keylistresult.error());
-        Q_ASSERT(keys.size() == 1);
+        QVERIFY(!keylistresult.error());
+        QVERIFY(keys.size() == 1);
         delete listjob;
 
         auto job = openpgp()->encryptJob(/*ASCII Armor */false, /* Textmode */ false);
-        Q_ASSERT(job);
+        QVERIFY(job);
         QByteArray plainBa;
         plainBa.fill('X', PROGRESS_TEST_SIZE);
         QByteArray cipherText;
 
         bool initSeen = false;
         bool finishSeen = false;
-        connect(job, &Job::progress, this, [this, &initSeen, &finishSeen] (const QString& what, int current, int total) {
+        connect(job, &Job::progress, this, [this, &initSeen, &finishSeen] (const QString&, int current, int total) {
                 // We only check for progress 0 and max progress as the other progress
                 // lines depend on the system speed and are as such unreliable to test.
-                Q_ASSERT(total == PROGRESS_TEST_SIZE);
+                QVERIFY(total == PROGRESS_TEST_SIZE);
                 if (current == 0) {
                     initSeen = true;
                 }
                 if (current == total) {
                     finishSeen = true;
                 }
-                Q_ASSERT(current >= 0 && current <= total);
+                QVERIFY(current >= 0 && current <= total);
             });
-        connect(job, &EncryptJob::result, this, [this, &initSeen, &finishSeen] (const GpgME::EncryptionResult &result,
-                                                                                const QByteArray &cipherText,
+        connect(job, &EncryptJob::result, this, [this, &initSeen, &finishSeen] (const GpgME::EncryptionResult &,
+                                                                                const QByteArray &,
                                                                                 const QString,
                                                                                 const GpgME::Error) {
-                Q_ASSERT(initSeen);
-                Q_ASSERT(finishSeen);
+                QVERIFY(initSeen);
+                QVERIFY(finishSeen);
                 Q_EMIT asyncDone();
             });
 
@@ -141,57 +169,126 @@ private Q_SLOTS:
 
         job->start(keys, inptr, outptr, Context::AlwaysTrust);
         QSignalSpy spy (this, SIGNAL(asyncDone()));
-        Q_ASSERT(spy.wait());
+        QVERIFY(spy.wait(QSIGNALSPY_TIMEOUT));
     }
 
     void testSymmetricEncryptDecrypt()
     {
-        auto ctx = Context::createForProtocol(OpenPGP);
+        if (!decryptSupported()) {
+            return;
+        }
+        auto job = openpgp()->encryptJob();
+        auto ctx = Job::context(job);
         TestPassphraseProvider provider;
         ctx->setPassphraseProvider(&provider);
         ctx->setPinentryMode(Context::PinentryLoopback);
         ctx->setArmor(true);
         ctx->setTextMode(true);
-        auto job = new QGpgMEEncryptJob(ctx);
         QByteArray cipherText;
         auto result = job->exec(std::vector<Key>(), QStringLiteral("Hello symmetric World").toUtf8(), Context::AlwaysTrust, cipherText);
         delete job;
-        Q_ASSERT(!result.error());
+        QVERIFY(!result.error());
         const auto cipherString = QString::fromUtf8(cipherText);
-        Q_ASSERT(cipherString.startsWith("-----BEGIN PGP MESSAGE-----"));
+        QVERIFY(cipherString.startsWith("-----BEGIN PGP MESSAGE-----"));
 
         killAgent(mDir.path());
 
-        auto ctx2 = Context::createForProtocol(OpenPGP);
+        auto decJob = openpgp()->decryptJob();
+        auto ctx2 = Job::context(decJob);
         ctx2->setPassphraseProvider(&provider);
         ctx2->setPinentryMode(Context::PinentryLoopback);
-        auto decJob = new QGpgMEDecryptJob(ctx2);
         QByteArray plainText;
         auto decResult = decJob->exec(cipherText, plainText);
-        Q_ASSERT(!result.error());
-        Q_ASSERT(QString::fromUtf8(plainText) == QStringLiteral("Hello symmetric World"));
+        QVERIFY(!result.error());
+        QVERIFY(QString::fromUtf8(plainText) == QStringLiteral("Hello symmetric World"));
         delete decJob;
     }
 
+    void testEncryptDecryptNowrap()
+    {
+        /* Now decrypt */
+        if (!decryptSupported()) {
+            return;
+        }
+        auto listjob = openpgp()->keyListJob(false, false, false);
+        std::vector<Key> keys;
+        auto keylistresult = listjob->exec(QStringList() << QStringLiteral("alfa@example.net"),
+                                          false, keys);
+        QVERIFY(!keylistresult.error());
+        QVERIFY(keys.size() == 1);
+        delete listjob;
+
+        auto job = openpgp()->signEncryptJob(/*ASCII Armor */true, /* Textmode */ true);
+
+        auto encSignCtx = Job::context(job);
+        TestPassphraseProvider provider1;
+        encSignCtx->setPassphraseProvider(&provider1);
+        encSignCtx->setPinentryMode(Context::PinentryLoopback);
+
+        QVERIFY(job);
+        QByteArray cipherText;
+        auto result = job->exec(keys, keys, QStringLiteral("Hello World").toUtf8(), Context::AlwaysTrust, cipherText);
+        delete job;
+        QVERIFY(!result.first.error());
+        QVERIFY(!result.second.error());
+        const auto cipherString = QString::fromUtf8(cipherText);
+        QVERIFY(cipherString.startsWith("-----BEGIN PGP MESSAGE-----"));
+
+        /* Now decrypt */
+        if (!decryptSupported()) {
+            return;
+        }
+
+        auto decJob = openpgp()->decryptJob();
+        auto ctx = Job::context(decJob);
+        TestPassphraseProvider provider;
+        ctx->setPassphraseProvider(&provider);
+        ctx->setPinentryMode(Context::PinentryLoopback);
+        ctx->setDecryptionFlags(Context::DecryptUnwrap);
+
+        QByteArray plainText;
+        auto decResult = decJob->exec(cipherText, plainText);
+
+        QVERIFY(!decResult.error());
+
+        delete decJob;
+
+        // Now verify the unwrapeped data.
+        auto verifyJob = openpgp()->verifyOpaqueJob(true);
+        QByteArray verified;
+
+        auto verResult = verifyJob->exec(plainText, verified);
+        QVERIFY(!verResult.error());
+        delete verifyJob;
+
+        QVERIFY(verResult.numSignatures() == 1);
+        auto sig = verResult.signatures()[0];
+
+        QVERIFY(verified == QStringLiteral("Hello World"));
+    }
+
 private:
     /* Loopback and passphrase provider don't work for mixed encryption.
      * So this test is disabled until gnupg(?) is fixed for this. */
     void testMixedEncryptDecrypt()
     {
+        if (!decryptSupported()) {
+            return;
+        }
         auto listjob = openpgp()->keyListJob(false, false, false);
         std::vector<Key> keys;
         auto keylistresult = listjob->exec(QStringList() << QStringLiteral("alfa@example.net"),
                                           false, keys);
-        Q_ASSERT(!keylistresult.error());
-        Q_ASSERT(keys.size() == 1);
+        QVERIFY(!keylistresult.error());
+        QVERIFY(keys.size() == 1);
         delete listjob;
 
-        auto ctx = Context::createForProtocol(OpenPGP);
+        auto job = openpgp()->encryptJob();
+        auto ctx = Job::context(job);
         ctx->setPassphraseProvider(new TestPassphraseProvider);
         ctx->setPinentryMode(Context::PinentryLoopback);
         ctx->setArmor(true);
         ctx->setTextMode(true);
-        auto job = new QGpgMEEncryptJob(ctx);
         QByteArray cipherText;
         printf("Before exec, flags: %x\n", Context::Symmetric | Context::AlwaysTrust);
         auto result = job->exec(keys, QStringLiteral("Hello symmetric World").toUtf8(),
@@ -199,31 +296,31 @@ private:
                                 cipherText);
         printf("After exec\n");
         delete job;
-        Q_ASSERT(!result.error());
+        QVERIFY(!result.error());
         printf("Cipher:\n%s\n", cipherText.constData());
         const auto cipherString = QString::fromUtf8(cipherText);
-        Q_ASSERT(cipherString.startsWith("-----BEGIN PGP MESSAGE-----"));
+        QVERIFY(cipherString.startsWith("-----BEGIN PGP MESSAGE-----"));
 
         killAgent(mDir.path());
 
-        /* Now create a new homedir which with we test symetric decrypt. */
+        /* Now create a new homedir which with we test symmetric decrypt. */
         QTemporaryDir tmp;
         qputenv("GNUPGHOME", tmp.path().toUtf8());
         QFile agentConf(tmp.path() + QStringLiteral("/gpg-agent.conf"));
-        Q_ASSERT(agentConf.open(QIODevice::WriteOnly));
+        QVERIFY(agentConf.open(QIODevice::WriteOnly));
         agentConf.write("allow-loopback-pinentry");
         agentConf.close();
 
-        auto ctx2 = Context::createForProtocol(OpenPGP);
+        auto decJob = openpgp()->decryptJob();
+        auto ctx2 = Job::context(decJob);
         ctx2->setPassphraseProvider(new TestPassphraseProvider);
         ctx2->setPinentryMode(Context::PinentryLoopback);
         ctx2->setTextMode(true);
-        auto decJob = new QGpgMEDecryptJob(ctx2);
         QByteArray plainText;
         auto decResult = decJob->exec(cipherText, plainText);
-        Q_ASSERT(!decResult.error());
+        QVERIFY(!decResult.error());
         qDebug() << "Plain: " << plainText;
-        Q_ASSERT(QString::fromUtf8(plainText) == QStringLiteral("Hello symmetric World"));
+        QVERIFY(QString::fromUtf8(plainText) == QStringLiteral("Hello symmetric World"));
         delete decJob;
 
         killAgent(tmp.path());
@@ -237,12 +334,12 @@ public Q_SLOT:
         QGpgMETest::initTestCase();
         const QString gpgHome = qgetenv("GNUPGHOME");
         qputenv("GNUPGHOME", mDir.path().toUtf8());
-        Q_ASSERT(mDir.isValid());
+        QVERIFY(mDir.isValid());
         QFile agentConf(mDir.path() + QStringLiteral("/gpg-agent.conf"));
-        Q_ASSERT(agentConf.open(QIODevice::WriteOnly));
+        QVERIFY(agentConf.open(QIODevice::WriteOnly));
         agentConf.write("allow-loopback-pinentry");
         agentConf.close();
-        Q_ASSERT(copyKeyrings(gpgHome, mDir.path()));
+        QVERIFY(copyKeyrings(gpgHome, mDir.path()));
     }
 
 private:
