@@ -639,7 +639,7 @@ add_io_cb (engine_g13_t g13, iocb_data_t *iocbd, gpgme_io_cb_t handler)
   gpgme_error_t err;
 
   TRACE_BEG  (DEBUG_ENGINE, "engine-g13:add_io_cb", g13,
-              "fd %d, dir %d", iocbd->fd, iocbd->dir);
+              "fd=%d, dir %d", iocbd->fd, iocbd->dir);
   err = (*g13->io_cbs.add) (g13->io_cbs.add_priv,
 			      iocbd->fd, iocbd->dir,
 			      handler, iocbd->data, &iocbd->tag);
