@@ -1,21 +1,22 @@
 /* gpgme-tool.c - Assuan server exposing GnuPG Made Easy operations.
-   Copyright (C) 2009, 2010, 2012, 2013 g10 Code GmbH
-   Copyright (C) 2001, 2003, 2009, 2011 Free Software Foundation, Inc.
-
-   This file is part of GPGME.
-
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
-   (at your option) any later version.
-
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * Copyright (C) 2009, 2010, 2012, 2013 g10 Code GmbH
+ * Copyright (C) 2001, 2003, 2009, 2011 Free Software Foundation, Inc.
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute this file and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 #if HAVE_CONFIG_H
@@ -3101,7 +3102,7 @@ cmd_hash_algo_name (assuan_context_t ctx, char *line)
 
 
 static const char hlp_identify[] =
-  "IDENTIY\n"
+  "IDENTIFY\n"
   "\n"
   "Identify the type of data set with the INPUT command.";
 static gpg_error_t
@@ -3282,15 +3283,11 @@ gpgme_server (gpgme_tool_t gt)
   gt->write_data_hook = &server;
 
   /* We use a pipe based server so that we can work from scripts.
-     assuan_init_pipe_server will automagically detect when we are
-     called with a socketpair and ignore FIELDES in this case. */
-#ifdef HAVE_W32CE_SYSTEM
-  filedes[0] = ASSUAN_STDIN;
-  filedes[1] = ASSUAN_STDOUT;
-#else
+   * assuan_init_pipe_server will automagically detect when we are
+   * called with a socketpair and ignore FILEDES in this case. */
   filedes[0] = assuan_fdopen (0);
   filedes[1] = assuan_fdopen (1);
-#endif
+
   err = assuan_new (&server.assuan_ctx);
   if (err)
     log_error (1, err, "can't create assuan context");
@@ -3439,10 +3436,5 @@ main (int argc, char *argv[])
   if (needgt)
     gpgme_release (gt.ctx);
 
-#ifdef HAVE_W32CE_SYSTEM
-  /* Give the buggy ssh server time to flush the output buffers.  */
-  Sleep (300);
-#endif
-
   return 0;
 }
