@@ -1,23 +1,23 @@
 /* key.c - Key objects.
-   Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004 g10 Code GmbH
-
-   This file is part of GPGME.
- 
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-   
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-   
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+ * Copyright (C) 2000 Werner Koch (dd9jn)
+ * Copyright (C) 2001, 2002, 2003, 2004 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -30,6 +30,9 @@
 #include "util.h"
 #include "ops.h"
 #include "sema.h"
+#include "debug.h"
+#include "mbox-util.h"
+
 
 
 /* Protects all reference counters in keys.  All other accesses to a
@@ -45,7 +48,7 @@ _gpgme_key_new (gpgme_key_t *r_key)
 
   key = calloc (1, sizeof *key);
   if (!key)
-    return gpg_error_from_errno (errno);
+    return gpg_error_from_syserror ();
   key->_refs = 1;
 
   *r_key = key;
@@ -60,7 +63,7 @@ _gpgme_key_add_subkey (gpgme_key_t key, gpgme_subkey_t *r_subkey)
 
   subkey = calloc (1, sizeof *subkey);
   if (!subkey)
-    return gpg_error_from_errno (errno);
+    return gpg_error_from_syserror ();
   subkey->keyid = subkey->_keyid;
   subkey->_keyid[16] = '\0';
 
@@ -78,7 +81,7 @@ _gpgme_key_add_subkey (gpgme_key_t key, gpgme_subkey_t *r_subkey)
 static char *
 set_user_id_part (char *tail, const char *buf, size_t len)
 {
-  while (len && (buf[len - 1] == ' ' || buf[len - 1] == '\t')) 
+  while (len && (buf[len - 1] == ' ' || buf[len - 1] == '\t'))
     len--;
   for (; len; len--)
     *tail++ = *buf++;
@@ -157,10 +160,10 @@ parse_user_id (char *src, char **name, char **email,
 	{
 	  in_name = 1;
 	  start = src;
-	}    
+	}
       src++;
     }
- 
+
   if (in_name)
     {
       if (!*name)
@@ -169,7 +172,7 @@ parse_user_id (char *src, char **name, char **email,
 	  tail = set_user_id_part (tail, start, src - start);
 	}
     }
- 
+
   /* Let unused parts point to an EOS.  */
   tail--;
   if (!*name)
@@ -187,7 +190,7 @@ parse_x509_user_id (char *src, char **name, char **email,
 {
   if (*src == '<' && src[strlen (src) - 1] == '>')
     *email = src;
-  
+
   /* Let unused parts point to an EOS.  */
   tail--;
   if (!*name)
@@ -214,7 +217,7 @@ _gpgme_key_append_name (gpgme_key_t key, const char *src, int convert)
      size, so that we are able to store the parsed stuff there too.  */
   uid = malloc (sizeof (*uid) + 2 * src_len + 3);
   if (!uid)
-    return gpg_error_from_errno (errno);
+    return gpg_error_from_syserror ();
   memset (uid, 0, sizeof *uid);
 
   uid->uid = ((char *) uid) + sizeof (*uid);
@@ -232,6 +235,16 @@ _gpgme_key_append_name (gpgme_key_t key, const char *src, int convert)
     parse_user_id (uid->uid, &uid->name, &uid->email,
 		   &uid->comment, dst);
 
+  uid->address = _gpgme_mailbox_from_userid (uid->uid);
+  if ((!uid->email || !*uid->email) && uid->address && uid->name
+      && !strcmp (uid->name, uid->address))
+    {
+      /* Name and address are the same. This is a mailbox only key.
+         Use address as email and remove name. */
+      *uid->name = '\0';
+      uid->email = uid->address;
+    }
+
   if (!key->uids)
     key->uids = uid;
   if (key->_last_uid)
@@ -279,7 +292,7 @@ _gpgme_key_add_sig (gpgme_key_t key, char *src)
 		       &sig->comment, dst);
     }
   else
-    sig->uid = '\0';
+    sig->uid[0] = '\0';
 
   if (!uid->signatures)
     uid->signatures = sig;
@@ -328,10 +341,10 @@ gpgme_key_unref (gpgme_key_t key)
   while (subkey)
     {
       gpgme_subkey_t next = subkey->next;
-      if (subkey->fpr)
-	free (subkey->fpr);
-      if (subkey->card_number)
-	free (subkey->card_number);
+      free (subkey->fpr);
+      free (subkey->curve);
+      free (subkey->keygrip);
+      free (subkey->card_number);
       free (subkey);
       subkey = next;
     }
@@ -341,6 +354,7 @@ gpgme_key_unref (gpgme_key_t key)
     {
       gpgme_user_id_t next_uid = uid->next;
       gpgme_key_sig_t keysig = uid->signatures;
+      gpgme_tofu_info_t tofu = uid->tofu;
 
       while (keysig)
 	{
@@ -358,21 +372,32 @@ gpgme_key_unref (gpgme_key_t key)
           free (keysig);
 	  keysig = next_keysig;
         }
+
+      while (tofu)
+        {
+          /* NB: The ->next is currently not used but we are prepared
+           * for it.  */
+          gpgme_tofu_info_t tofu_next = tofu->next;
+
+          free (tofu->description);
+          free (tofu);
+          tofu = tofu_next;
+        }
+
+      free (uid->address);
       free (uid);
       uid = next_uid;
     }
-  
-  if (key->issuer_serial)
-    free (key->issuer_serial);
-  if (key->issuer_name)
-    free (key->issuer_name);
 
-  if (key->chain_id)
-    free (key->chain_id);
+  free (key->issuer_serial);
+  free (key->issuer_name);
+  free (key->chain_id);
+  free (key->fpr);
 
   free (key);
 }
 
+
 
 /* Support functions.  */
 
@@ -518,7 +543,7 @@ gpgme_key_get_string_attr (gpgme_key_t key, _gpgme_attr_t what,
     case GPGME_ATTR_FPR:
       return subkey ? subkey->fpr : NULL;
 
-    case GPGME_ATTR_ALGO:    
+    case GPGME_ATTR_ALGO:
       return subkey ? gpgme_pubkey_algo_name (subkey->pubkey_algo) : NULL;
 
     case GPGME_ATTR_TYPE:
@@ -527,10 +552,10 @@ gpgme_key_get_string_attr (gpgme_key_t key, _gpgme_attr_t what,
     case GPGME_ATTR_OTRUST:
       return otrust_to_string (key->owner_trust);
 
-    case GPGME_ATTR_USERID:  
+    case GPGME_ATTR_USERID:
       return uid ? uid->uid : NULL;
 
-    case GPGME_ATTR_NAME:   
+    case GPGME_ATTR_NAME:
       return uid ? uid->name : NULL;
 
     case GPGME_ATTR_EMAIL:
@@ -542,7 +567,7 @@ gpgme_key_get_string_attr (gpgme_key_t key, _gpgme_attr_t what,
     case GPGME_ATTR_VALIDITY:
       return uid ? validity_to_string (uid->validity) : NULL;
 
-    case GPGME_ATTR_KEY_CAPS:    
+    case GPGME_ATTR_KEY_CAPS:
       return subkey ? capabilities_to_string (subkey) : NULL;
 
     case GPGME_ATTR_SERIAL:
@@ -604,7 +629,7 @@ gpgme_key_get_ulong_attr (gpgme_key_t key, _gpgme_attr_t what,
       return (subkey && subkey->timestamp >= 0)
 	? (unsigned long) subkey->timestamp : 0;
 
-    case GPGME_ATTR_EXPIRE: 
+    case GPGME_ATTR_EXPIRE:
       return (subkey && subkey->expires >= 0)
 	? (unsigned long) subkey->expires : 0;
 
@@ -693,13 +718,13 @@ gpgme_key_sig_get_string_attr (gpgme_key_t key, int uid_idx,
     case GPGME_ATTR_KEYID:
       return certsig->keyid;
 
-    case GPGME_ATTR_ALGO:    
+    case GPGME_ATTR_ALGO:
       return gpgme_pubkey_algo_name (certsig->pubkey_algo);
 
     case GPGME_ATTR_USERID:
       return certsig->uid;
 
-    case GPGME_ATTR_NAME:   
+    case GPGME_ATTR_NAME:
       return certsig->name;
 
     case GPGME_ATTR_EMAIL:
@@ -707,7 +732,7 @@ gpgme_key_sig_get_string_attr (gpgme_key_t key, int uid_idx,
 
     case GPGME_ATTR_COMMENT:
       return certsig->comment;
-   
+
     default:
       return NULL;
     }
@@ -725,13 +750,13 @@ gpgme_key_sig_get_ulong_attr (gpgme_key_t key, int uid_idx, _gpgme_attr_t what,
 
   switch (what)
     {
-    case GPGME_ATTR_ALGO:    
+    case GPGME_ATTR_ALGO:
       return (unsigned long) certsig->pubkey_algo;
 
-    case GPGME_ATTR_CREATED: 
+    case GPGME_ATTR_CREATED:
       return certsig->timestamp < 0 ? 0L : (unsigned long) certsig->timestamp;
 
-    case GPGME_ATTR_EXPIRE: 
+    case GPGME_ATTR_EXPIRE:
       return certsig->expires < 0 ? 0L : (unsigned long) certsig->expires;
 
     case GPGME_ATTR_KEY_REVOKED:
